#version 460 core

layout (location = 0) in vec3 pos;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 uv;

struct PerInstanceData
{
	mat4 modelMatrix;
	vec3 albedoTint;
	uint bindlessTextureHandleIdx;
};

layout (std430, binding = 0) readonly buffer PerInstanceBlock
{
	PerInstanceData perInstanceData[];
};

out VS_OUT {
    vec4 FragPos;
    vec3 Normal;
    vec2 TexCoords;
	flat vec3 AlbedoTint;
	flat uint BindlessTextureHandleIdx;
} vs_out;

out gl_PerVertex
{
  vec4 gl_Position;
};

uniform mat4 viewproj;

void main()
{
	mat4 modelMatrix = perInstanceData[gl_DrawID].modelMatrix;
    vs_out.FragPos = modelMatrix * vec4(pos, 1.0);
    vs_out.Normal = normalize(modelMatrix * vec4(normal,0)).xyz;
    vs_out.TexCoords = uv;
    vs_out.BindlessTextureHandleIdx = perInstanceData[gl_DrawID].bindlessTextureHandleIdx;
	gl_Position = viewproj * vs_out.FragPos;
}
